﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;

namespace XamDataChartOtherSeriesBinderDemo
{
    public class MainViewModel : PropertyChangedNotifier
    {
        private ObservableCollection<SeriesViewModel> seriesSource;
        private ObservableCollection<SeriesViewModel> seriesSource2;

        public MainViewModel()
        {
            SeriesSource = new ObservableCollection<SeriesViewModel>
            {
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(10),
                    Type = SeriesType.ColumnSeries,
                    XPath = "Date",
                    YPath = "Value",
                },
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(22),
                    Type = SeriesType.LineSeries,
                    XPath = "Label",
                    YPath = "Value",
                },

                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(20),
                    Type = SeriesType.ScatterSeries,
                    XPath = "Index",
                    YPath = "Value",
                },
                new SeriesViewModel
                {
                    Source = ChartData.GenerateData(15),
                    Type = SeriesType.ScatterLineSeries,
                    XPath = "Index",
                    YPath = "Value",
                },
            };

            SeriesSource2 = new ObservableCollection<SeriesViewModel>();
        }

        public ObservableCollection<SeriesViewModel> SeriesSource
        {
            get { return seriesSource; }
            set { seriesSource = value; OnPropertyChagned("SeriesSource"); }
        }

        public ObservableCollection<SeriesViewModel> SeriesSource2
        {
            get { return seriesSource2; }
            set { seriesSource2 = value; OnPropertyChagned("SeriesSource2"); }
        }

        public ICommand AddSeries
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>(
                        (obj) =>
                        {
                            SeriesSource2.Add(
                                new SeriesViewModel
                                {
                                    Source = new CosXPlusCosY(),
                                    Type = SeriesType.ScatterContourSeries,
                                    XPath = "X",
                                    YPath = "Y",
                                    ValueMemberPath = "Z"
                                });

                            SeriesSource.Add(
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(10),
                                    Type = SeriesType.ScatterLineSeries,
                                    XPath = "Index",
                                    YPath = "Value",
                                    ValueMemberPath = "Value"
                                });
                        }));
            }
        }

        public ICommand RemoveSeries
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>(
                        (obj) =>
                        {
                            if (SeriesSource.Count > 0)
                                SeriesSource.RemoveAt(0);
                        }));
            }
        }

        public ICommand ClearSeries
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>(
                        (obj) =>
                        {
                            SeriesSource.Clear();
                        }));
            }
        }

        public ICommand ResetSeries
        {
            get
            {
                return new RelayCommand<object>(
                    new Action<object>(
                        (obj) =>
                        {
                            SeriesSource = new ObservableCollection<SeriesViewModel>
                            {
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(10),
                                    Type = SeriesType.ColumnSeries,
                                    XPath = "Date",
                                    YPath = "Value",
                                },
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(22),
                                    Type = SeriesType.LineSeries,
                                    XPath = "Label",
                                    YPath = "Value",
                                },

                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(20),
                                    Type = SeriesType.ScatterSeries,
                                    XPath = "Index",
                                    YPath = "Value",
                                },
                                new SeriesViewModel
                                {
                                    Source = ChartData.GenerateData(15),
                                    Type = SeriesType.ScatterLineSeries,
                                    XPath = "Index",
                                    YPath = "Value",
                                },
                            };
                        }));
            }
        }
    }



    public class ChartData
    {
        public DateTime Date { get; set; }
        public int Index { get; set; }
        public string Label { get; set; }
        public double Value { get; set; }

        public static ObservableCollection<ChartData> GenerateData(int count)
        {
            ObservableCollection<ChartData> data = new ObservableCollection<ChartData>();

            Random r = new Random();

            ChartData data0 = new ChartData
            {
                Date = DateTime.Now,
                Index = 0,
                Value = r.Next(20, 40),
                Label = "Item 0"
            };

            data.Add(data0);

            for (int i = 1; i < count; i++)
            {
                ChartData dat = new ChartData
                {
                    Date = DateTime.Now.AddDays(i),
                    Index = i,
                    Label = "Item " + i,
                };

                if (r.NextDouble() > .5)
                    dat.Value = data[i - 1].Value + r.NextDouble();
                else
                    dat.Value = data[i - 1].Value - r.NextDouble();

                data.Add(dat);
            }

            return data;
        }
    }
}
