﻿using Infragistics.Controls.Charts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace XamDataChartOtherSeriesBinderDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = new MainViewModel();
        }
    }
   
    public class PropertyChangedNotifier : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChagned(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }

    public enum SeriesType
    {
        LineSeries,
        ColumnSeries,
        ScatterSeries,
        ScatterLineSeries,
        ScatterContourSeries
    }

    public class RelayCommand<T> : ICommand
    {

        #region Declarations

        readonly Predicate<T> _canExecute;
        readonly Action<T> _execute;

        #endregion

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class and the command can always be executed.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        public RelayCommand(Action<T> execute)
            : this(execute, null)
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="RelayCommand&lt;T&gt;"/> class.
        /// </summary>
        /// <param name="execute">The execution logic.</param>
        /// <param name="canExecute">The execution status logic.</param>
        public RelayCommand(Action<T> execute, Predicate<T> canExecute)
        {

            if (execute == null)
                throw new ArgumentNullException("execute");
            _execute = execute;
            _canExecute = canExecute;
        }

        #endregion

        #region ICommand Members

        public event EventHandler CanExecuteChanged
        {
            add
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested += value;
            }
            remove
            {

                if (_canExecute != null)
                    CommandManager.RequerySuggested -= value;
            }
        }

        [DebuggerStepThrough]
        public virtual Boolean CanExecute(Object parameter)
        {
            return _canExecute == null ? true : _canExecute((T)parameter);
        }

        public virtual void Execute(Object parameter)
        {
            _execute((T)parameter);
        }

        #endregion
    }

    public abstract class Surface3D : ObservableCollection<ShapePoint3D>
    {
        public Surface3D()
            : this(11, 11)
        {
        }
        public Surface3D(int xCount, int yCount)
            : this(-100, 100, -100, 100, xCount, yCount)
        {
        }
        public Surface3D(double xMin, double xMax,
                         double yMin, double yMax,
                         int xCount = 11, int yCount = 11)
        {
            XMin = xMin;
            XMax = xMax;
            YMin = xMin;
            YMax = yMax;
            XCount = xCount;
            YCount = yCount;
            Generate();
        }

        public double XMin { get; private set; }
        public double XMax { get; private set; }
        public double YMin { get; private set; }
        public double YMax { get; private set; }
        public double XCount { get; private set; }
        public double YCount { get; private set; }
        protected void Generate()
        {
            var xStep = (XMax - XMin) / (XCount - 1);
            var yStep = (YMax - YMin) / (YCount - 1);
            for (var x = XMin; x <= XMax; x += xStep)
            {
                for (var y = YMin; y <= YMax; y += yStep)
                {
                    this.Add(new ShapePoint3D { X = x, Y = y, Z = this.Z(x, y) });
                }
            }
        }
        protected abstract new double Z(double x, double y);
    }

    public class CosXPlusCosY : Surface3D
    {
        protected override double Z(double x, double y)
        {
            return Math.Cos(x) + Math.Cos(y);
        }
    }

    public class ShapePoint3D
    {
        public double X { get; set; }
        public double Y { get; set; }
        public double Z { get; set; }
    }

}
