﻿using Infragistics.Controls.Charts;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Data;

namespace XamDataChartOtherSeriesBinderDemo
{
    public class SeriesBinderInfo : FrameworkElement
    {
        #region Fields
        private XamDataChart _owner;
        Dictionary<object, Series> _seriesObjectMapper;
        #endregion

        #region Constructor

        public SeriesBinderInfo()
        {
            _seriesObjectMapper = new Dictionary<object, Series>();
        }

        #endregion

        #region Initializing Methods

        public void AttachOwner(XamDataChart chart)
        {
            if (_owner != null)
            {
                DetachOwner(_owner);
            }
            _owner = chart;
            SetBinding(DataContextProperty,
                new Binding("DataContext")
                {
                    Source = _owner
                });

        }

        public void DetachOwner(XamDataChart _owner)
        {
            _owner = null;
            _seriesObjectMapper.Clear();
            ClearValue(DataContextProperty);
        }

        #endregion  

        #region Properties

        #region SeriesSource

        private const string SeriesSourcePropertyName = "SeriesSource";

        public IEnumerable SeriesSource
        {
            get { return (IEnumerable)GetValue(SeriesSourceProperty); }
            set { SetValue(SeriesSourceProperty, value); }
        }

        public static readonly DependencyProperty SeriesSourceProperty =
            DependencyProperty.Register(
                SeriesSourcePropertyName,
                typeof(IEnumerable),
                typeof(SeriesBinderInfo),
                new PropertyMetadata(null));

        #endregion

        #region ItemsSourcePath

        private const string ItemsSourcePathPropertyName = "ItemsSourcePath";

        public string ItemsSourcePath
        {
            get { return (string)GetValue(ItemsSourcePathProperty); }
            set { SetValue(ItemsSourcePathProperty, value); }
        }

        public static readonly DependencyProperty ItemsSourcePathProperty =
            DependencyProperty.Register(
                ItemsSourcePathPropertyName,
                typeof(string),
                typeof(SeriesBinderInfo),
                new PropertyMetadata(null));


        #endregion

        #region XMemberPath

        private const string XMemberPathPropertyName = "XMemberPath";

        public string XMemberPath
        {
            get { return (string)GetValue(XMemberPathProperty); }
            set { SetValue(XMemberPathProperty, value); }
        }

        public static readonly DependencyProperty XMemberPathProperty =
            DependencyProperty.Register(
                    XMemberPathPropertyName,
                    typeof(string),
                    typeof(SeriesBinderInfo),
                    new PropertyMetadata(null));

        #endregion

        #region YMemberPath

        private const string YMemberPathPropertyName = "YMemberPath";

        public string YMemberPath
        {
            get { return (string)GetValue(YMemberPathProperty); }
            set { SetValue(YMemberPathProperty, value); }
        }

        public static readonly DependencyProperty YMemberPathProperty =
            DependencyProperty.Register(
                    YMemberPathPropertyName,
                    typeof(string),
                    typeof(SeriesBinderInfo),
                    new PropertyMetadata(null));

        #endregion

        #region valueMemberPath

        private const string ValueMemberPathPropertyName = "ValueMemberPath";

        public string ValueMemberPath
        {
            get { return (string)GetValue(ValueMemberPathProperty); }
            set { SetValue(ValueMemberPathProperty, value); }
        }

        public static readonly DependencyProperty ValueMemberPathProperty =
            DependencyProperty.Register(
                    ValueMemberPathPropertyName,
                    typeof(string),
                    typeof(SeriesBinderInfo),
                    new PropertyMetadata(null));

        #endregion

        #region TypePath

        private const string TypePathPropertyName = "TypePath";

        public string TypePath
        {
            get { return (string)GetValue(TypePathProperty); }
            set { SetValue(TypePathProperty, value); }
        }

        public static readonly DependencyProperty TypePathProperty =
            DependencyProperty.Register(
                TypePathPropertyName,
                typeof(string),
                typeof(SeriesBinderInfo),
                new PropertyMetadata(null));



        #endregion

        #endregion

        #region Event Handlers

        void SeriesBinderInfo_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    {
                        object seriesSource = e.NewItems[0];
                        AddSeries(seriesSource);
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    {
                        object seriesSource = e.OldItems[0];
                        RemoveSeries(seriesSource);
                    }
                    break;
                case NotifyCollectionChangedAction.Move:
                case NotifyCollectionChangedAction.Replace:
                case NotifyCollectionChangedAction.Reset:
                    {
                        GenerateSeries();
                    }
                    break;
                default:
                    break;
            }
        }

        #endregion

        #region Helper Methods

        private void RemoveSeries(object seriesSource)
        {
            Series s = _seriesObjectMapper[seriesSource];
            SeriesType type = (SeriesType)seriesSource.GetType().GetProperty(TypePath).GetValue(seriesSource, null);

            UpdateSeriesProperties(s, type, seriesSource, true, true);
            _owner.Series.Remove(s);
            _seriesObjectMapper.Remove(seriesSource);
        }

        private void AddSeries(object seriesSource)
        {
            SeriesType type = (SeriesType)seriesSource.GetType().GetProperty(TypePath).GetValue(seriesSource, null);

            Series series = CreateSeries(type, seriesSource);
            _owner.Series.Add(series);

            _seriesObjectMapper.Add(seriesSource, series);
        }

        private void GenerateSeries()
        {
            _owner.Series.Clear();
            _owner.Axes.Clear();
            foreach (var item in SeriesSource)
            {
                AddSeries(item);
            }
        }

        private Series CreateSeries(SeriesType type, object seriesSource)
        {
            Series series = null;
            switch (type)
            {
                case SeriesType.LineSeries:
                    {
                        series = new LineSeries();
                        UpdateSeriesProperties(series, type, seriesSource, true, false);
                    }
                    break;
                case SeriesType.ColumnSeries:
                    {
                        series = new ColumnSeries();
                        UpdateSeriesProperties(series, type, seriesSource, true, false);
                    }
                    break;
                case SeriesType.ScatterSeries:
                    {
                        series = new ScatterSeries();
                        UpdateSeriesProperties(series, type, seriesSource, true, false);
                    }
                    break;
                case SeriesType.ScatterLineSeries:
                    {
                        series = new ScatterLineSeries();
                        UpdateSeriesProperties(series, type, seriesSource, true, false);
                    }
                    break;
                case SeriesType.ScatterContourSeries:
                    {
                        series = new ScatterContourSeries();
                        UpdateSeriesProperties(series, type, seriesSource, true, false);

                    }

                    break;
                default:
                    break;
            }

            return series;
        }

        private void UpdateSeriesProperties(Series series, SeriesType type, object seriesSource, bool shouldRenewAxes, bool clearOnly)
        {
            if (series != null)
            {
                switch (type)
                {
                    case SeriesType.LineSeries:
                    case SeriesType.ColumnSeries:
                        {
                            HorizontalAnchoredCategorySeries category = series as HorizontalAnchoredCategorySeries;
                            category.ClearValue(HorizontalAnchoredCategorySeries.ValueMemberPathProperty);
                            category.ClearValue(HorizontalAnchoredCategorySeries.ItemsSourceProperty);

                            if (!clearOnly)
                            {
                                category.SetBinding(HorizontalAnchoredCategorySeries.ValueMemberPathProperty,
                                    new Binding(YMemberPath) { Source = seriesSource });

                                category.SetBinding(HorizontalAnchoredCategorySeries.ItemsSourceProperty,
                                    new Binding(ItemsSourcePath) { Source = seriesSource });
                            }

                            if (shouldRenewAxes)
                            {
                                if (category.XAxis != null)
                                    _owner.Axes.Remove(category.XAxis);

                                if (category.YAxis != null)
                                    _owner.Axes.Remove(category.YAxis);

                                if (!clearOnly)
                                {
                                    CategoryXAxis xAxis = new CategoryXAxis();
                                    xAxis.SetBinding(CategoryXAxis.ItemsSourceProperty,
                                        new Binding(ItemsSourcePath) { Source = seriesSource });
                                    xAxis.Label = "{" + seriesSource.GetType().GetProperty(XMemberPath).GetValue(seriesSource, null) + "}";

                                    NumericYAxis yAxis = new NumericYAxis();

                                    _owner.Axes.Add(xAxis);
                                    _owner.Axes.Add(yAxis);

                                    category.XAxis = xAxis;
                                    category.YAxis = yAxis;
                                }
                            }
                        }
                        break;
                    case SeriesType.ScatterSeries:
                    case SeriesType.ScatterLineSeries:
                        {
                            ScatterBase scatter = series as ScatterBase;
                            scatter.ClearValue(ScatterBase.XMemberPathProperty);
                            scatter.ClearValue(ScatterBase.YMemberPathProperty);
                            scatter.ClearValue(ScatterBase.ItemsSourceProperty);

                            if (!clearOnly)
                            {
                                scatter.SetBinding(ScatterBase.XMemberPathProperty,
                                    new Binding(XMemberPath) { Source = seriesSource });

                                scatter.SetBinding(ScatterBase.YMemberPathProperty,
                                    new Binding(YMemberPath) { Source = seriesSource });

                                scatter.SetBinding(ScatterBase.ItemsSourceProperty,
                                    new Binding(ItemsSourcePath) { Source = seriesSource });
                            }


                            if (shouldRenewAxes)
                            {
                                if (scatter.XAxis != null)
                                    _owner.Axes.Remove(scatter.XAxis);

                                if (scatter.YAxis != null)
                                    _owner.Axes.Remove(scatter.YAxis);

                                if (!clearOnly)
                                {
                                    NumericXAxis xAxis = new NumericXAxis();
                                    NumericYAxis yAxis = new NumericYAxis();

                                    _owner.Axes.Add(xAxis);
                                    _owner.Axes.Add(yAxis);

                                    scatter.XAxis = xAxis;
                                    scatter.YAxis = yAxis;
                                }
                            }
                        }
                        break;

                    case SeriesType.ScatterContourSeries:
                        {
                            ScatterContourSeries scatterContour = series as ScatterContourSeries;
                            scatterContour.ClearValue(ScatterTriangulationSeries.XMemberPathProperty);
                            scatterContour.ClearValue(ScatterTriangulationSeries.YMemberPathProperty);
                            scatterContour.ClearValue(ScatterContourSeries.ValueMemberPathProperty);
                            scatterContour.ClearValue(ScatterTriangulationSeries.ItemsSourceProperty);

                            if (!clearOnly)
                            {
                                scatterContour.SetBinding(ScatterTriangulationSeries.XMemberPathProperty,
                                    new Binding(XMemberPath) { Source = seriesSource });

                                scatterContour.SetBinding(ScatterTriangulationSeries.YMemberPathProperty,
                                    new Binding(YMemberPath) { Source = seriesSource });

                                scatterContour.SetBinding(ScatterContourSeries.ValueMemberPathProperty,
                                   new Binding("ValueMemberPath") { Source = seriesSource });

                                scatterContour.SetBinding(ScatterTriangulationSeries.ItemsSourceProperty,
                                    new Binding(ItemsSourcePath) { Source = seriesSource });
                            }


                            if (shouldRenewAxes)
                            {
                                if (scatterContour.XAxis != null)
                                    _owner.Axes.Remove(scatterContour.XAxis);

                                if (scatterContour.YAxis != null)
                                    _owner.Axes.Remove(scatterContour.YAxis);

                                if (!clearOnly)
                                {
                                    NumericXAxis xAxis = new NumericXAxis();
                                    NumericYAxis yAxis = new NumericYAxis();


                                    _owner.Axes.Add(xAxis);
                                    _owner.Axes.Add(yAxis);

                                    scatterContour.XAxis = xAxis;
                                    scatterContour.YAxis = yAxis;
                                    // scatterContour.ValueMemberPath = 
                                }
                            }
                            ;
                        }
                        break;

                    default:
                        break;
                }

            }

        }

        #endregion

        #region Base Overrides

        protected override void OnPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            switch (e.Property.Name)
            {
                case SeriesSourcePropertyName:
                    {
                        if (e.OldValue != null
                            && e.OldValue is INotifyCollectionChanged)
                            (e.OldValue as INotifyCollectionChanged).CollectionChanged -= SeriesBinderInfo_CollectionChanged;
                        if (e.NewValue != null
                            && e.NewValue is INotifyCollectionChanged)
                            (e.NewValue as INotifyCollectionChanged).CollectionChanged += SeriesBinderInfo_CollectionChanged;

                        GenerateSeries();
                    }
                    break;
                case XMemberPathPropertyName:
                case YMemberPathPropertyName:
                case ValueMemberPathPropertyName:
                case TypePathPropertyName:
                    {
                        if (SeriesSource != null)
                            GenerateSeries();
                    }
                    break;
                default:
                    break;
            }
            base.OnPropertyChanged(e);
        }

        #endregion
    }
}
